import controlP5.*;

// Hold on to some user interface stuff.  Mainly just
// create a text field to hold the number to visualize.
ControlP5 cp5;
Textfield txt;

void setup()
{
  size( 500, 500 );
  noStroke();
  fill( 0 );

  cp5 = new ControlP5( this );
  txt = cp5.addTextfield( "Number" )
    .setText( "45" )
    .setSize( 50, 30 )
    .setPosition( 10, 10 )
    .setAutoClear( false )
    .setFocus( true );
}

// A (recursive!!) helpful function to generate
// a list of a number's prime factors.  This helper
// function doesn't check for 2 as a factor -- it 
// turns out that grouping 2s into 4s makes for
// prettier diagrams.  Don't call this function
// directly.
int[] primeFactorsNoTwos( int n )
{
  for ( int k = 3; k < n; ++k ) {
    if ( (n%k) == 0 ) {
      int[] rec = primeFactorsNoTwos( n / k );
      return append( rec, k );
    }
  }

  return new int[] { n };
}

// Use this function to get the array of prime factors
// associated with a number.  It plays a little trick
// by treating 4 as "prime", which makes prettier diagrams.
int[] getPrimeFactors( int n )
{
  int twos = 0;
  int ch = n;
  while( ch%2 == 0 ) {
    ++twos;
    ch = ch / 2;
  }
  
  if( twos % 2 == 1 ) {
    return append( primeFactorsNoTwos( n/2 ), 2 );
  } else {
    return primeFactorsNoTwos( n );
  }
}

// Given an array of integers, peel off the first
// element of the array and return a smaller array 
// of everything else.  Very useful for writing down
// a recursive call in this sketch.
int[] restOfArray( int[] arr )
{
  int[] narr = new int[ arr.length - 1 ];
  for( int idx = 0; idx < narr.length; ++idx ) {
    narr[idx] = arr[idx+1];
  }
  return narr;
}

// If you want to place K circles (or recursive
// sub-rings!) inside a unit circle, this is how
// far from the centre of the unit circle each of
// the centres of those sub-circles will be. The
// radius of each sub-circle will be one minus
// this number, to make sure it reaches the edge of
// the outer circle.
float subRingRadius( int K )
{
  return lerp( 1 / (1 + sin(PI/K)), 1, 0.2 );
}

// Write a recursive function that takes as input an
// array of integers.  Check for the base case of an 
// array of size zero, in which case you should draw a
// unit circle.  If the array is non-empty, check
// the first element of the array and draw a ring 
// of that many sub-rings, each of which is a transformed
// copy (using geometric context) of a recursive call to
// the same function.  The body of this function will be
// about 12-15 lines of code.

void draw()
{
  background( 255 );

  pushMatrix();
  
  // Translate and scale so that a unit circle will 
  // be drawn at the right location and size.
  translate( width/2, height/2 );
  scale( 0.9*width/2 );
  
  // Get the number currently in the text field.
  int num = int( txt.getText() );
  if ( num > 0 ) {
    // Call your recursive function here, passing in 
    // the list of the number's prime factors.
  }
  popMatrix();
}

